@@ -1,4 +1,4 @@
-/* $Id: gweather-applet.c,v 1.96 2005/07/10 04:40:34 davyd Exp $ */
+/* $Id: gweather-applet.c,v 1.100 2005/12/12 00:22:17 philipl Exp $ */
 
 /*
  *  Papadimitriou Spiros <spapadim+@cs.cmu.edu>
@@ -27,7 +27,6 @@
 #include <libnotify/notify.h>
 #endif
 
-#include "weather.h"
 #include "gweather.h"
 #include "gweather-about.h"
 #include "gweather-pref.h"
@@ -219,7 +218,6 @@ key_press_cb (GtkWidget *widget, GdkEventKey *event, GWeatherApplet *gw_applet)
 	case GDK_KP_Space:
 		gweather_dialog_open(gw_applet);
 		return TRUE;
-		break;
 	default:
 		break;
 	}
@@ -308,30 +306,13 @@ applet_destroy (GtkWidget *widget, GWeatherApplet *gw_applet)
        gw_applet->timeout_tag = 0;
     }
 	
-    if (gw_applet->gweather_info->metar_handle) {
-       gnome_vfs_async_cancel(gw_applet->gweather_info->metar_handle);
-       gw_applet->gweather_info->metar_handle = NULL;
+    if (gw_applet->gconf) {
+       gweather_gconf_free (gw_applet->gconf);
     }
 
-    if (gw_applet->gweather_info->iwin_handle) {
-       gnome_vfs_async_cancel(gw_applet->gweather_info->iwin_handle);
-       gw_applet->gweather_info->iwin_handle = NULL;
-    }
+    weather_info_abort (gw_applet->gweather_info);
 
-    if (gw_applet->gweather_info->wx_handle) {
-       gnome_vfs_async_cancel(gw_applet->gweather_info->wx_handle);
-       gw_applet->gweather_info->wx_handle = NULL;
-    }
-
-    if (gw_applet->gweather_info->met_handle) {
-       gnome_vfs_async_cancel(gw_applet->gweather_info->met_handle);
-       gw_applet->gweather_info->met_handle = NULL;
-    }
-
-    if (gw_applet->gweather_info->bom_handle) {
-       gnome_vfs_async_cancel(gw_applet->gweather_info->bom_handle);
-       gw_applet->gweather_info->bom_handle = NULL;
-    }
+    gtk_main_quit ();
 }
 
 void gweather_applet_create (GWeatherApplet *gw_applet)
@@ -421,26 +402,32 @@ gint timeout_cb (gpointer data)
     return 0;  /* Do not repeat timeout (will be re-set by gweather_update) */
 }
 
-void update_finish (WeatherInfo *info)
+static void
+update_finish (WeatherInfo *info, gpointer data)
 {
     static int gw_fault_counter = 0;
 #ifdef HAVE_LIBNOTIFY
     static NotifyIcon *icon = NULL;
+    NotifyHints *hints;
     char *notification_message, *notification_detail;
     GdkPixbuf *pixbuf = NULL;
     GConfClient *conf;
+    int x, y;
+    GtkRequisition size;
 #endif
     char *s;
-    GWeatherApplet *gw_applet = info->applet;
-    
-    if (FALSE == info->valid)
-    {
-	    /* there has been an error during retrival
-	     * just update the fault counter
-	     */
-	    gw_fault_counter++;
-    }
-    else if ((TRUE == info->valid) ||
+    GWeatherApplet *gw_applet = (GWeatherApplet *)data;
+   
+    /* Update timer */
+    if (gw_applet->timeout_tag > 0)
+        gtk_timeout_remove(gw_applet->timeout_tag);
+    if (gw_applet->gweather_pref.update_enabled)
+        gw_applet->timeout_tag =  
+        	gtk_timeout_add (
+			gw_applet->gweather_pref.update_interval * 1000,
+                        timeout_cb, gw_applet);
+ 
+    if ((TRUE == weather_info_is_valid (info)) ||
 	     (gw_fault_counter >= MAX_CONSECUTIVE_FAULTS))
     {
 	    gw_fault_counter = 0;
@@ -458,15 +445,6 @@ void update_finish (WeatherInfo *info)
 				    gw_applet->applet), s, NULL);
 	    g_free (s);
 	    
-	    /* Update timer */
-	    if (gw_applet->timeout_tag > 0)
-	        gtk_timeout_remove(gw_applet->timeout_tag);
-	    if (gw_applet->gweather_pref.update_enabled)
-	        gw_applet->timeout_tag =  
-	        	gtk_timeout_add (
-				gw_applet->gweather_pref.update_interval * 1000,
-	                        timeout_cb, gw_applet);
-
 	    /* Update dialog -- if one is present */
 	    gweather_dialog_update(gw_applet);
 	    
@@ -506,12 +484,21 @@ void update_finish (WeatherInfo *info)
                 g_free (tmp);
             }
             
+	    gdk_window_get_origin (GTK_WIDGET (gw_applet->applet)->window, &x, &y);
+	    gtk_widget_size_request (GTK_WIDGET (gw_applet->applet), &size);
+	    x += size.width / 2;
+	    y += size.height;
+
+	    hints = notify_hints_new ();
+	    notify_hints_set_int (hints, "x", x);
+	    notify_hints_set_int (hints, "y", y);
+	    
             /* Show notification */
             notification_message = g_strdup_printf ("%s: %s",
-                                                    weather_info_get_location (info),
+                                                    weather_info_get_location_name (info),
                                                     weather_info_get_sky (info));
             notification_detail = g_strdup_printf (_("City: %s\nSky: %s\nTemperature: %s"),
-            		                           weather_info_get_location (info),
+            		                           weather_info_get_location_name (info),
             		                           weather_info_get_sky (info),
             		                           weather_info_get_temp_summary (info));
             
@@ -521,7 +508,7 @@ void update_finish (WeatherInfo *info)
 	                                   notification_detail,	/* body text */
 	                                   icon,		/* icon */
 	                                   TRUE, 0,		/* expiry, server default */
-	                                   NULL,		/* hints */
+	                                   hints,		/* hints */
 	                                   NULL,		/* no user_data */
 	                                   0))			/* no actions */
 	        g_warning ("Could not send notification to daemon\n");
@@ -530,11 +517,18 @@ void update_finish (WeatherInfo *info)
 	    g_free (notification_detail);
 #endif
     }
+    else
+    {
+	    /* there has been an error during retrival
+	     * just update the fault counter
+	     */
+	    gw_fault_counter++;
+    }
 }
 
 void gweather_update (GWeatherApplet *gw_applet)
 {
-    gboolean update_success;
+    WeatherPrefs prefs;
 
     weather_info_get_pixbuf_mini(gw_applet->gweather_info, 
     				 &(gw_applet->applet_pixbuf));
@@ -545,23 +539,30 @@ void gweather_update (GWeatherApplet *gw_applet)
     			 _("Updating..."), NULL);
 
     /* Set preferred forecast type */
-    weather_forecast_set(gw_applet->gweather_pref.detailed ? 
-    			 	FORECAST_ZONE : FORECAST_STATE);
+    prefs.type = gw_applet->gweather_pref.detailed ? FORECAST_ZONE : FORECAST_STATE;
 
     /* Set radar map retrieval option */
-    weather_radar_set(gw_applet->gweather_pref.radar_enabled);
+    prefs.radar = gw_applet->gweather_pref.radar_enabled;
+    prefs.radar_custom_url = (gw_applet->gweather_pref.use_custom_radar_url &&
+    				gw_applet->gweather_pref.radar) ?
+				gw_applet->gweather_pref.radar : NULL;
+
+    /* Set the units */
+    prefs.temperature_unit = gw_applet->gweather_pref.temperature_unit;
+    prefs.speed_unit = gw_applet->gweather_pref.speed_unit;
+    prefs.pressure_unit = gw_applet->gweather_pref.pressure_unit;
+    prefs.distance_unit = gw_applet->gweather_pref.distance_unit;
 
     /* Update current conditions */
     if (gw_applet->gweather_info && 
-    	weather_location_equal(gw_applet->gweather_info->location, 
+    	weather_location_equal(weather_info_get_location(gw_applet->gweather_info),
     			       gw_applet->gweather_pref.location)) {
-        update_success = weather_info_update(gw_applet, 
-        				      gw_applet->gweather_info, 
-        				      update_finish);
+	weather_info_update(gw_applet->gweather_info, &prefs,
+			    update_finish, gw_applet);
     } else {
         weather_info_free(gw_applet->gweather_info);
-        gw_applet->gweather_info = NULL;
-        update_success = weather_info_new((gpointer)gw_applet, 
-        		 gw_applet->gweather_pref.location, update_finish);
+        gw_applet->gweather_info = weather_info_new(gw_applet->gweather_pref.location,
+						    &prefs,
+						    update_finish, gw_applet);
     }
 }
