@@ -1,3 +1,4 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 2 -*- */
 /* GNOME Volume Applet
  * Copyright (C) 2004 Ronald Bultje <rbultje@ronald.bitfreak.net>
  *
@@ -23,11 +24,25 @@
 #include "config.h"
 #endif
 
+#include <math.h>
 #include <string.h>
 
-#include <libgnomeui/libgnomeui.h>
+#include <glib-object.h>
 #include <gdk/gdkkeysyms.h>
-#include <gtk/gtk.h>
+
+#include <gtk/gtkaboutdialog.h>
+#include <gtk/gtkbutton.h>
+#include <gtk/gtkicontheme.h>
+#include <gtk/gtkmain.h>
+#include <gtk/gtkmessagedialog.h>
+#include <gtk/gtkrange.h>
+#include <gtk/gtktooltips.h>
+#include <gtk/gtkwidget.h>
+
+#include <gconf/gconf-client.h>
+
+#include <libgnomeui/gnome-help.h>
+#include <libgnome/gnome-desktop-item.h>
 
 #include "applet.h"
 #include "keys.h"
@@ -49,6 +64,8 @@ static gboolean	gnome_volume_applet_button	(GtkWidget *widget,
 						 GdkEventButton *event);
 static gboolean	gnome_volume_applet_key		(GtkWidget *widget,
 						 GdkEventKey *event);
+static gdouble  gnome_volume_applet_get_volume  (GstMixer *mixer, 
+						 GstMixerTrack *track);
 
 static void	gnome_volume_applet_background	(PanelApplet *panel_applet,
 						 PanelAppletBackgroundType type,
@@ -59,7 +76,8 @@ static void	gnome_volume_applet_orientation	(PanelApplet *applet,
 static void	gnome_volume_applet_size	(PanelApplet *applet,
 						 guint      size);
 
-static void	gnome_volume_applet_refresh	(GnomeVolumeApplet *applet);
+static void	gnome_volume_applet_refresh	(GnomeVolumeApplet *applet,
+						 gboolean           force_refresh);
 static gboolean	cb_check			(gpointer   data);
 
 static void	cb_volume			(GtkAdjustment *adj,
@@ -79,17 +97,19 @@ static void	cb_ui_event			(BonoboUIComponent *comp,
 						 Bonobo_UIComponent_EventType type,
 						 const gchar       *state_string,
 						 gpointer           data);
+static void	cb_theme_change                (GtkIconTheme *icon_theme,
+						gpointer      data);
 
 static PanelAppletClass *parent_class = NULL;
 static struct {
   gchar *filename;
   GdkPixbuf *pixbuf;
 } pix[] = {
-  { GNOME_PIXMAPSDIR "/mixer/volume-mute.png",   NULL },
-  { GNOME_PIXMAPSDIR "/mixer/volume-zero.png",   NULL },
-  { GNOME_PIXMAPSDIR "/mixer/volume-min.png",    NULL },
-  { GNOME_PIXMAPSDIR "/mixer/volume-medium.png", NULL },
-  { GNOME_PIXMAPSDIR "/mixer/volume-max.png",    NULL },
+  { "stock_volume-mute", NULL },
+  { "stock_volume-0",    NULL },
+  { "stock_volume-min",  NULL },
+  { "stock_volume-med",  NULL },
+  { "stock_volume-max",  NULL },
   { NULL, NULL }
 };
 
@@ -121,44 +141,28 @@ gnome_volume_applet_get_type (void)
   return gnome_volume_applet_type;
 }
 
-/*
- * Hi, I'm slow.
- */
-
-static inline void
-flip_byte (gchar *one, gchar *two)
-{
-  gint temp;
-
-  temp = *one;
-  *one = *two;
-  *two = temp;
-}
-
-static inline void
-flip_pixel (gchar *line, gint pixel, gint width, gint bpp)
+static void
+init_pixbufs (GnomeVolumeApplet *applet)
 {
   gint n;
+  
+  for (n = 0; pix[n].filename != NULL; n++) {
+    if (pix[n].pixbuf)
+      g_object_unref (pix[n].pixbuf);
+    
+    pix[n].pixbuf = gtk_icon_theme_load_icon (
+      	    applet->icon_theme,
+      	    pix[n].filename,
+      	    panel_applet_get_size (&applet->parent),
+      	    0,
+      	    NULL);
+    if (gtk_widget_get_default_direction () == GTK_TEXT_DIR_RTL) {
+      GdkPixbuf *temp;
 
-  for (n = 0; n < bpp; n++) {
-    flip_byte (&line[pixel * bpp + n], &line[(width - 1 - pixel) * bpp + n]);
-  }
-}
-
-static void
-flip (GdkPixbuf *pix)
-{
-  gint w = gdk_pixbuf_get_width (pix),
-       h = gdk_pixbuf_get_height (pix),
-       x, y, stride = gdk_pixbuf_get_rowstride (pix),
-       bpp = gdk_pixbuf_get_n_channels (pix);
-  gchar *data = gdk_pixbuf_get_pixels (pix);
-
-  for (y = 0; y < h; y++) {
-    for (x = 0; x < (w / 2); x++) {
-      flip_pixel (data, x, w, bpp);
+      temp = gdk_pixbuf_flip (pix[n].pixbuf, TRUE);
+      g_object_unref (G_OBJECT (pix[n].pixbuf));
+      pix[n].pixbuf = temp;
     }
-    data += stride;
   }
 }
 
@@ -168,7 +172,6 @@ gnome_volume_applet_class_init (GnomeVolumeAppletClass *klass)
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   GtkWidgetClass *gtkwidget_class = GTK_WIDGET_CLASS (klass);
   PanelAppletClass *panelapplet_class = PANEL_APPLET_CLASS (klass);
-  gint n;
 
   parent_class = g_type_class_ref (PANEL_TYPE_APPLET);
 
@@ -183,45 +186,46 @@ gnome_volume_applet_class_init (GnomeVolumeAppletClass *klass)
   /* FIXME:
    * - style-set.
    */
-
-  /* init pixbufs */
-  for (n = 0; pix[n].filename != NULL; n++) {
-    pix[n].pixbuf = gdk_pixbuf_new_from_file (pix[n].filename, NULL);
-    if (gtk_widget_get_default_direction () == GTK_TEXT_DIR_RTL) {
-      flip (pix[n].pixbuf);
-    }
-  }
 }
 
 static void
 gnome_volume_applet_init (GnomeVolumeApplet *applet)
 {
-  GtkWidget *image, *dock;
-  GtkTooltips *tooltips;
+  GtkWidget *image;
 
   applet->timeout = 0;
   applet->elements = NULL;
   applet->client = gconf_client_get_default ();
   applet->mixer = NULL;
-  applet->track = NULL;
+  applet->tracks = NULL;
   applet->lock = FALSE;
   applet->state = -1;
   applet->prefs = NULL;
+  applet->icon_theme = gtk_icon_theme_get_default ();
+
+  /* init pixbufs */
+  init_pixbufs (applet);
 
   /* icon (our main UI) */
   image = gtk_image_new ();
   applet->image = GTK_IMAGE (image);
   gtk_container_add (GTK_CONTAINER (applet), image);
   gtk_widget_show (image);
+  gtk_window_set_default_icon_name ("volume-knob");
 
   /* dock window (expanded UI) */
   applet->pop = FALSE;
 
   /* tooltip over applet */
-  tooltips = gtk_tooltips_new ();
-  gtk_tooltips_set_tip (tooltips, GTK_WIDGET (applet),
+  applet->tooltips = gtk_tooltips_new ();
+  gtk_tooltips_set_tip (applet->tooltips, GTK_WIDGET (applet),
 			_("Volume Control"), NULL);
 
+  /* handle icon theme changes */
+  g_signal_connect (G_OBJECT (applet->icon_theme),
+		    "changed", G_CALLBACK (cb_theme_change),
+		    applet);
+
   /* other stuff */
   panel_applet_add_preferences (PANEL_APPLET (applet),
 				"/schemas/apps/mixer_applet/prefs",
@@ -230,16 +234,136 @@ gnome_volume_applet_init (GnomeVolumeApplet *applet)
 			  PANEL_APPLET_EXPAND_MINOR);
 }
 
-void
+/* Parse the list of tracks that are stored in GConf */
+
+static char **
+parse_track_list (const char *track_list)
+{
+  if (track_list)
+    return g_strsplit (track_list, ":", 0);
+  else
+    return NULL;
+}
+
+static GList *
+select_tracks (GstElement *element,
+	      const char *active_track_names,
+	      gboolean    reset_state)
+{
+  const GList *tracks, *l;
+  GstMixerTrack *track_fallback;
+  GList *active_tracks;
+  char **active_track_name_list;
+
+  active_tracks = NULL;
+  track_fallback = NULL;
+  active_track_name_list = NULL;
+
+  if (reset_state) {
+    if (gst_element_set_state (element, GST_STATE_READY) != GST_STATE_SUCCESS)
+      return NULL;
+  }
+
+  tracks = gst_mixer_list_tracks (GST_MIXER (element));
+  if (active_track_names)
+    active_track_name_list = parse_track_list (active_track_names);
+
+  for (l = tracks; l; l = l->next) {
+    GstMixerTrack *track = l->data;
+    gint i;
+
+    if (!track->num_channels)
+      continue;
+
+    if (!track_fallback)
+      track_fallback = track;
+
+    if (GST_MIXER_TRACK_HAS_FLAG (track, GST_MIXER_TRACK_MASTER))
+      track_fallback = track;
+
+    if (active_track_name_list) {
+      for (i = 0; active_track_name_list[i] != NULL; i++) {
+	gchar *track_test = active_track_name_list[i];
+	
+	if (!strcmp (track_test, track->label))
+	  active_tracks = g_list_append (active_tracks, track);
+      }
+    }
+  }
+
+  /* if the list had no matches and we've got a fallback track,
+   * then use it. */
+
+  if (!active_tracks && track_fallback)
+    active_tracks = g_list_append (active_tracks, track_fallback);
+    
+  if (!active_tracks && reset_state) {
+    gst_element_set_state (element, GST_STATE_NULL);
+  }
+
+  g_strfreev (active_track_name_list);
+  return active_tracks;
+}
+
+static gboolean
+select_element_and_track (GnomeVolumeApplet *applet,
+			  GList             *elements,
+			  const char        *active_element_name,
+			  const char        *active_track_names)
+{
+  GstElement *active_element;
+  GList *active_tracks, *l;
+
+  applet->elements = elements;
+
+  active_element = NULL;
+  active_tracks = NULL;
+
+  if (active_element_name) {
+    for (l = elements; l; l = l->next) {
+      GstElement *element = l->data;
+      const char *element_name;
+
+      element_name = g_object_get_data (G_OBJECT (element),
+				      "gnome-volume-applet-name");
+
+      if (!strcmp (element_name, active_element_name)) {
+	active_element = element;
+	break;
+      }
+    }
+  }
+
+  if (active_element)
+    active_tracks = select_tracks (active_element, active_track_names, TRUE);
+
+  if (!active_tracks) {
+    active_element = NULL;
+    for (l = elements; l; l = l->next) {
+      GstElement *element = l->data;
+
+      if ((active_tracks = select_tracks (element, active_track_names, TRUE))) {
+	active_element = element;
+	break;
+      }
+    }
+  }
+  
+  if (!active_element)
+    return FALSE;
+
+  applet->mixer = g_object_ref (active_element);
+  applet->tracks = active_tracks;
+  g_list_foreach (applet->tracks, (GFunc) g_object_ref, NULL);
+
+  return TRUE;
+}
+
+gboolean
 gnome_volume_applet_setup (GnomeVolumeApplet *applet,
 			   GList *elements)
 {
-  GConfValue *value;
   GtkObject *adj;
-  GstElement *active_element;
-  GstMixerTrack *active_track;
-  const GList *item;
-  gint page;
   BonoboUIComponent *component;
   static const BonoboUIVerb verbs[] = {
     BONOBO_UI_UNSAFE_VERB ("RunMixer", cb_verb),
@@ -249,85 +373,44 @@ gnome_volume_applet_setup (GnomeVolumeApplet *applet,
     BONOBO_UI_UNSAFE_VERB ("Pref",     cb_verb),
     BONOBO_UI_VERB_END
   };
+  gchar *key;
+  gchar *active_element_name;
+  gchar *active_track_name;
+  GstMixerTrack *first_track;
+  gboolean res;
 
-  /* default element to first */
-  g_return_if_fail (elements != NULL);
-  active_element = elements->data;
+  active_element_name = panel_applet_gconf_get_string (PANEL_APPLET (applet),
+						       GNOME_VOLUME_APPLET_KEY_ACTIVE_ELEMENT,
+						       NULL);
 
-  applet->elements = elements;
+  active_track_name = panel_applet_gconf_get_string (PANEL_APPLET (applet),
+						     GNOME_VOLUME_APPLET_KEY_ACTIVE_TRACK,
+						     NULL);
 
-  /* get active element, if any (otherwise we use the default) */
-  if ((value = gconf_client_get (applet->client,
-				 GNOME_VOLUME_APPLET_KEY_ACTIVE_ELEMENT,
-				 NULL)) != NULL &&
-      value->type == GCONF_VALUE_STRING) {
-    const gchar *active_el_str, *cur_el_str;
+  res = select_element_and_track (applet, elements, active_element_name,
+				  active_track_name);
+  g_free (active_element_name);
+  g_free (active_track_name);
 
-    active_el_str = gconf_value_get_string (value);
-    for (item = elements; item != NULL; item = item->next) {
-      cur_el_str = g_object_get_data (item->data, "gnome-volume-applet-name");
-      if (!strcmp (active_el_str, cur_el_str)) {
-        active_element = item->data;
-        break;
-      }
-    }
+  if (res) {
+    first_track = g_list_first (applet->tracks)->data;
+
+    /* tell the dock */
+    adj = gtk_adjustment_new (50, 0, 100, 4, 10, 0);
+    gtk_adjustment_set_value (GTK_ADJUSTMENT (adj), 
+			      gnome_volume_applet_get_volume (applet->mixer, 
+							      first_track));
   }
 
-  /* use the active element */
-  gst_element_set_state (active_element, GST_STATE_READY);
-  applet->mixer = GST_MIXER (active_element);
-  gst_object_ref (GST_OBJECT (active_element));
-
-  /* default track: first */
-  item = gst_mixer_list_tracks (applet->mixer);
-  active_track = item->data;
-  /* FIXME:
-   * - what if first has 0 channels?
-   */
-
-  /* get active track, if any (otherwise we use the default) */
-  if ((value = gconf_client_get (applet->client,
-				 GNOME_VOLUME_APPLET_KEY_ACTIVE_TRACK,
-				 NULL)) != NULL &&
-      value->type == GCONF_VALUE_STRING) {
-    const gchar *track_name = gconf_value_get_string (value);
-
-    for ( ; item != NULL; item = item->next) {
-      GstMixerTrack *track = item->data;
-
-      if (!track->num_channels)
-        continue;
-
-      if (!strcmp (track_name, track->label)) {
-        active_track = item->data;
-        break;
-      } else if (!track &&
-		 GST_MIXER_TRACK_HAS_FLAG (track,
-					   GST_MIXER_TRACK_MASTER)) {
-        active_track = item->data;
-      }
-    }
-  }
-
-  /* use the active track */
-  applet->track = active_track;
-  g_object_ref (G_OBJECT (active_track));
-
-  /* tell the dock */
-  page = (applet->track->max_volume - applet->track->min_volume) / 10;
-  adj = gtk_adjustment_new (0, applet->track->min_volume,
-			    applet->track->max_volume,
-			    (page / 5 > 0) ? page / 5 : 1,
-			    page, 0);
   gnome_volume_applet_orientation (PANEL_APPLET (applet),
 				   panel_applet_get_orient (PANEL_APPLET (applet)));
-  gnome_volume_applet_dock_change (applet->dock,
-				   GTK_ADJUSTMENT (adj));
-  g_signal_connect (adj, "value-changed",
-		    G_CALLBACK (cb_volume), applet);
 
-  gnome_volume_applet_refresh (applet);
-  applet->timeout = g_timeout_add (100, cb_check, applet);
+  if (res) {
+    gnome_volume_applet_dock_change (applet->dock,
+				     GTK_ADJUSTMENT (adj));
+    g_signal_connect (adj, "value-changed",
+		      G_CALLBACK (cb_volume), applet);
+  }
 
   /* menu - done here because bonobo is intialized now */
   panel_applet_setup_menu_from_file (PANEL_APPLET (applet), 
@@ -337,13 +420,28 @@ gnome_volume_applet_setup (GnomeVolumeApplet *applet,
   component = panel_applet_get_popup_component (PANEL_APPLET (applet));
   g_signal_connect (component, "ui-event", G_CALLBACK (cb_ui_event), applet);
 
-  /* gconf */
-  gconf_client_add_dir (applet->client, GNOME_VOLUME_APPLET_KEY_DIR,
-			GCONF_CLIENT_PRELOAD_RECURSIVE, NULL);
-  gconf_client_notify_add (applet->client, GNOME_VOLUME_APPLET_KEY_DIR,
-			   cb_gconf, applet, NULL, NULL);
+  gnome_volume_applet_refresh (applet, TRUE);
+  if (res) {
+    applet->timeout = g_timeout_add (100, cb_check, applet);
+  }
+
+  if (res) {
+    /* gconf */
+    key = panel_applet_gconf_get_full_key (PANEL_APPLET (applet),
+				GNOME_VOLUME_APPLET_KEY_ACTIVE_ELEMENT);
+    gconf_client_notify_add (applet->client, key,
+			     cb_gconf, applet, NULL, NULL);
+    g_free (key);
+    key = panel_applet_gconf_get_full_key (PANEL_APPLET (applet),
+				GNOME_VOLUME_APPLET_KEY_ACTIVE_TRACK);
+    gconf_client_notify_add (applet->client, key,
+			     cb_gconf, applet, NULL, NULL);
+    g_free (key);
+  }
 
   gtk_widget_show (GTK_WIDGET (applet));
+
+  return TRUE;
 }
 
 static void
@@ -368,9 +466,10 @@ gnome_volume_applet_dispose (GObject *object)
     applet->elements = NULL;
   }
 
-  if (applet->track) {
-    g_object_unref (G_OBJECT (applet->track));
-    applet->track = NULL;
+  if (applet->tracks) {
+    g_list_foreach (applet->tracks, (GFunc) g_object_unref, NULL);
+    g_list_free (applet->tracks);
+    applet->tracks = NULL;
   }
 
   if (applet->mixer) {
@@ -386,6 +485,33 @@ gnome_volume_applet_dispose (GObject *object)
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
+/*
+ * Show a dialog (once) when no mixer is available.
+ */
+
+static void
+show_no_mixer_dialog (GnomeVolumeApplet *applet)
+{
+  static gboolean shown = FALSE;
+  GtkWidget *dialog;
+
+  if (shown)
+    return;
+  shown = TRUE;
+
+  dialog = gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_ERROR,
+		GTK_BUTTONS_CLOSE, "%s\n\n%s",
+		_("The volume control did not find any elements and/or "
+		  "devices to control. This means either that you don't "
+		  "have the right GStreamer plugins installed, or that you "
+		  "don't have a sound card configured."),
+		_("You can remove the volume control from the panel by "
+		  "right-clicking the speaker icon on the panel and "
+		  "selecting \"Remove From Panel\" from the menu."));
+  gtk_widget_show (dialog);
+  g_signal_connect (dialog, "response", G_CALLBACK (gtk_widget_destroy), NULL);
+}
+
 /*
  * get position that the dock should get based on applet position.
  */
@@ -444,7 +570,9 @@ gnome_volume_applet_popup_dock (GnomeVolumeApplet *applet)
   gint x, y;
 
   /* show (before reposition, so size allocation is done) */
-  gtk_widget_show (GTK_WIDGET (applet->dock));
+  gnome_volume_applet_get_dock_position (applet, &x, &y);
+  gtk_window_move (GTK_WINDOW (applet->dock), x, y);
+  gtk_widget_show_all (GTK_WIDGET (applet->dock));
 
   /* reposition */
   gnome_volume_applet_get_dock_position (applet, &x, &y);
@@ -481,7 +609,7 @@ gnome_volume_applet_popdown_dock (GnomeVolumeApplet *applet)
   gtk_grab_remove (widget);
 
   /* hide */
-  gtk_widget_hide (GTK_WIDGET (applet->dock));
+  gtk_widget_hide_all (GTK_WIDGET (applet->dock));
 
   /* set menu item as active */
   gtk_widget_set_state (GTK_WIDGET (applet), GTK_STATE_NORMAL);
@@ -508,10 +636,17 @@ static void
 gnome_volume_applet_toggle_mute (GnomeVolumeApplet *applet)
 {
   BonoboUIComponent *component;
-  gboolean mute = (GST_MIXER_TRACK_HAS_FLAG (applet->track,
-			GST_MIXER_TRACK_MUTE));
+  gboolean mute = applet->state & 1;
+  GtkAdjustment *adj = gtk_range_get_adjustment (applet->dock->scale);
+  GList *tracks;
 
-  gst_mixer_set_mute (applet->mixer, applet->track, !mute);
+  for (tracks = g_list_first (applet->tracks); tracks; tracks = tracks->next)
+    gst_mixer_set_mute (applet->mixer, tracks->data, !mute);
+
+  if (mute) {
+    /* sync back actual volume */
+    cb_volume (adj, applet);
+  }
 
   /* update component */
   component = panel_applet_get_popup_component (PANEL_APPLET (applet));
@@ -521,7 +656,7 @@ gnome_volume_applet_toggle_mute (GnomeVolumeApplet *applet)
 
   /* update graphic - this should happen automagically after the next
    * idle call, but apparently doesn't for some people... */
-  gnome_volume_applet_refresh (applet);
+  gnome_volume_applet_refresh (applet, TRUE);
 }
 
 /*
@@ -531,11 +666,22 @@ gnome_volume_applet_toggle_mute (GnomeVolumeApplet *applet)
 static void
 gnome_volume_applet_run_mixer (GnomeVolumeApplet *applet)
 {
+  GnomeDesktopItem *ditem;
   GError *error = NULL;
 
-  gdk_spawn_command_line_on_screen (
-      gtk_widget_get_screen (GTK_WIDGET (applet)),
-      "gnome-volume-control", &error);
+  if ((ditem = gnome_desktop_item_new_from_basename ("gnome-volume-control.desktop", 0, NULL))) {
+    gnome_desktop_item_set_launch_time (ditem, gtk_get_current_event_time ());
+    gnome_desktop_item_launch_on_screen (ditem, NULL,
+	                                 GNOME_DESKTOP_ITEM_LAUNCH_ONLY_ONE,
+					 gtk_widget_get_screen (GTK_WIDGET (applet)),
+					 -1, &error);
+    gnome_desktop_item_unref (ditem);
+  }
+  else {
+    gdk_spawn_command_line_on_screen (
+	      gtk_widget_get_screen (GTK_WIDGET (applet)),
+	      "gnome-volume-control", &error);
+  }
 
   if (error) {
     GtkWidget *dialog;
@@ -561,12 +707,17 @@ gnome_volume_applet_scroll (GtkWidget      *widget,
 {
   GnomeVolumeApplet *applet = GNOME_VOLUME_APPLET (widget);
 
+  if (!applet->mixer) {
+    show_no_mixer_dialog (applet);
+    return TRUE;
+  }
+
   if (event->type == GDK_SCROLL) {
     switch (event->direction) {
       case GDK_SCROLL_UP:
       case GDK_SCROLL_DOWN: {
         GtkAdjustment *adj = gtk_range_get_adjustment (applet->dock->scale);
-        gint volume = adj->value;
+        gdouble volume = adj->value;
 
         if (event->direction == GDK_SCROLL_UP) {
           volume += adj->step_increment;
@@ -586,7 +737,10 @@ gnome_volume_applet_scroll (GtkWidget      *widget,
     }
   }
 
-  return GTK_WIDGET_CLASS (parent_class)->scroll_event (widget, event);
+  if (GTK_WIDGET_CLASS (parent_class)->scroll_event)
+    return GTK_WIDGET_CLASS (parent_class)->scroll_event (widget, event);
+  else
+    return FALSE;
 }
 
 static gboolean
@@ -595,18 +749,25 @@ gnome_volume_applet_button (GtkWidget      *widget,
 {
   GnomeVolumeApplet *applet = GNOME_VOLUME_APPLET (widget);
 
-  if (event->window != GTK_WIDGET (applet)->window) {
+  if (event->window != GTK_WIDGET (applet)->window &&
+      event->type == GDK_BUTTON_PRESS) {
     gnome_volume_applet_popdown_dock (applet);
     return TRUE;
-  } else {
+  } else if (event->window == GTK_WIDGET (applet)->window) {
     switch (event->button) {
       case 1:
         switch (event->type) {
           case GDK_BUTTON_PRESS:
-            gnome_volume_applet_pop_dock (applet);
+            if (!applet->mixer) {
+              show_no_mixer_dialog (applet);
+            } else {
+              gnome_volume_applet_pop_dock (applet);
+            }
             return TRUE;
           case GDK_2BUTTON_PRESS:
-            gnome_volume_applet_popdown_dock (applet);
+            if (applet->mixer) {
+              gnome_volume_applet_popdown_dock (applet);
+            }
             gnome_volume_applet_run_mixer (applet);
             return TRUE;
           default:
@@ -637,7 +798,9 @@ gnome_volume_applet_key (GtkWidget   *widget,
 {
   GnomeVolumeApplet *applet = GNOME_VOLUME_APPLET (widget);
 
-  switch (event->keyval) {
+  if (!applet->mixer) {
+    show_no_mixer_dialog (applet);
+  } else switch (event->keyval) {
     case GDK_KP_Enter:
     case GDK_ISO_Enter:
     case GDK_3270_Enter:
@@ -666,7 +829,7 @@ gnome_volume_applet_key (GtkWidget   *widget,
     case GDK_Up:
     case GDK_Down: {
       GtkAdjustment *adj = gtk_range_get_adjustment (applet->dock->scale);
-      gint volume = adj->value, increment;
+      gdouble volume = adj->value, increment;
 
       if (event->keyval == GDK_Up || event->keyval == GDK_Down)
         increment = adj->step_increment;
@@ -730,7 +893,8 @@ static void
 gnome_volume_applet_size (PanelApplet *applet,
 			  guint        size)
 {
-  gnome_volume_applet_refresh (GNOME_VOLUME_APPLET (applet));
+  init_pixbufs (GNOME_VOLUME_APPLET (applet));
+  gnome_volume_applet_refresh (GNOME_VOLUME_APPLET (applet), TRUE);
 
   if (PANEL_APPLET_CLASS (parent_class)->change_size)
     PANEL_APPLET_CLASS (parent_class)->change_size (applet, size);
@@ -750,7 +914,7 @@ gnome_volume_applet_background (PanelApplet *_applet,
   gtk_widget_set_style (GTK_WIDGET (applet), NULL);
   rc_style = gtk_rc_style_new ();
   gtk_widget_modify_style (GTK_WIDGET (applet), rc_style);
-  g_object_unref (rc_style);
+  gtk_rc_style_unref (rc_style);
 
   switch (type) {
     case PANEL_NO_BACKGROUND:
@@ -765,10 +929,58 @@ gnome_volume_applet_background (PanelApplet *_applet,
         g_object_unref (style->bg_pixmap[GTK_STATE_NORMAL]);
       style->bg_pixmap[GTK_STATE_NORMAL] = g_object_ref (pixmap);
       gtk_widget_set_style (GTK_WIDGET (applet), style);
+      g_object_unref (style);
       break;
   }
 }
 
+/* 
+ * This needs to be here because not all tracks have the same volume range,
+ * so you can send this function the track and a new volume and it will be
+ * scaled according to the volume range of the track in question. 
+ */
+
+void
+gnome_volume_applet_adjust_volume (GstMixer *mixer,
+				   GstMixerTrack *track,
+				   gdouble volume)
+{
+  int range = track->max_volume - track->min_volume;
+  gdouble scale = ((gdouble) range) / 100;
+  int *volumes, n;
+
+  volume *= scale;
+  volume += track->min_volume;
+
+  volumes = g_new (gint, track->num_channels);
+  for (n = 0; n < track->num_channels; n++)
+    volumes[n] = lrint (volume);
+  gst_mixer_set_volume (mixer, track, volumes);
+  g_free (volumes);
+}
+
+static gdouble
+gnome_volume_applet_get_volume (GstMixer *mixer,
+				GstMixerTrack *track)
+{
+  int *volumes, n;
+  gdouble j;
+
+  if (!track || !mixer)
+    return -1;
+
+  volumes = g_new (gint, track->num_channels);
+  gst_mixer_get_volume (mixer, track, volumes);
+
+  j = 0;
+  for (n = 0; n < track->num_channels; n++)
+    j += volumes[n];
+  g_free (volumes);
+  j /= track->num_channels;
+
+  return 100 * j / (track->max_volume - track->min_volume);
+}
+
 /*
  * Volume changed.
  */
@@ -778,81 +990,114 @@ cb_volume (GtkAdjustment *adj,
 	   gpointer data)
 {
   GnomeVolumeApplet *applet = data;
-  gint *volumes, n;
+  gdouble v;
+  GList *iter;
 
   if (applet->lock)
     return;
   applet->lock = TRUE;
 
-  volumes = g_new (gint, applet->track->num_channels);
-  for (n = 0; n < applet->track->num_channels; n++)
-    volumes[n] = gtk_adjustment_get_value (adj);
-  gst_mixer_set_volume (applet->mixer, applet->track, volumes);
-  g_free (volumes);
+  v = gtk_adjustment_get_value (adj);
+
+  for (iter = g_list_first (applet->tracks); iter; iter = iter->next) {
+    GstMixerTrack *track = iter->data;
+    gnome_volume_applet_adjust_volume (applet->mixer, track, v);
+  }
 
   applet->lock = FALSE;
+  applet->force_next_update = TRUE;
 }
 
 /*
  * Automatic timer. Check for changes.
  */
 
-#define STATE(vol,m) ((vol << 1) | m)
+#define STATE(vol,m) (((gint) vol << 1) | (m ? 1 : 0))
 
 static void
-gnome_volume_applet_refresh (GnomeVolumeApplet *applet)
+gnome_volume_applet_refresh (GnomeVolumeApplet *applet,
+			     gboolean           force_refresh)
 {
   BonoboUIComponent *component;
-  GdkPixbuf *scaled, *orig;
-  guint size;
-  gint n, *volumes, volume = 0;
-  gboolean mute;
+  GdkPixbuf *pixbuf;
+  gint n;
+  gdouble volume;
+  gboolean mute, did_change;
+  gchar *tooltip_str;
+  GstMixerTrack *first_track;
+  GString *track_names;
+  GList *iter;
 
-  /* build-up phase */
-  if (!applet->track)
+  if (!applet->mixer) {
+    n = 0;
+    mute = FALSE;
+  } else if (!applet->tracks) {
     return;
+  } else {
+    /* only first track */
+    first_track = g_list_first (applet->tracks)->data;
+    volume = gnome_volume_applet_get_volume (applet->mixer, first_track);
+    mute = GST_MIXER_TRACK_HAS_FLAG (first_track,
+				     GST_MIXER_TRACK_MUTE);
+    if (volume <= 0)
+      mute = TRUE;
 
-  volumes = g_new (gint, applet->track->num_channels);
-  gst_mixer_get_volume (applet->mixer, applet->track, volumes);
-  for (n = 0; n < applet->track->num_channels; n++)
-    volume += volumes[n];
-  volume /= applet->track->num_channels;
-  g_free (volumes);
-  mute = GST_MIXER_TRACK_HAS_FLAG (applet->track,
-				   GST_MIXER_TRACK_MUTE);
-
-  n = 4 * volume / (applet->track->max_volume -
-      applet->track->min_volume) + 1;
-  if (n <= 0)
-    n = 1;
-  if (n >= 5)
-    n = 4;
-
-  if (STATE (n, mute) != applet->state) {
-    if (mute) {
-      orig = gdk_pixbuf_copy (pix[n].pixbuf);
-      gdk_pixbuf_composite (pix[0].pixbuf, orig, 0, 0,
-			    gdk_pixbuf_get_width (orig),
-			    gdk_pixbuf_get_height (orig),
-			    0, 0, 1.0, 1.0,
-			    GDK_INTERP_BILINEAR, 255);
-    } else {
-      orig = pix[n].pixbuf;
-    }
-
-    size = panel_applet_get_size (PANEL_APPLET (applet));
-    scaled = gdk_pixbuf_scale_simple (orig, size, size,
-				      GDK_INTERP_BILINEAR);
-    gtk_image_set_from_pixbuf (applet->image, scaled);
-    if (mute)
-      gdk_pixbuf_unref (orig);
-    gdk_pixbuf_unref (scaled);
-
-    applet->state = STATE (n, mute);
+    /* select image */
+    n = 4 * volume / 100 + 1;
+    if (n <= 0)
+      n = 1;
+    if (n >= 5)
+      n = 4;
   }
 
+  did_change = (force_refresh || (STATE (volume, mute) != applet->state) ||
+      applet->force_next_update);
+  applet->force_next_update = FALSE;
+
+  if (did_change) {
+    if (mute) {
+      pixbuf = pix[0].pixbuf;
+    } else {
+      pixbuf = pix[n].pixbuf;
+    }
+
+    gtk_image_set_from_pixbuf (applet->image, pixbuf);
+    applet->state = STATE (volume, mute);
+  }
+
+  if (!did_change || !applet->mixer)
+    return;
+
+  /* build names of selecter tracks */
+  track_names = g_string_new ("");
+  for (iter = g_list_first (applet->tracks); iter; iter = iter->next) {
+    GstMixerTrack *track = iter->data;
+
+    if (iter->next != NULL)
+      g_string_append_printf (track_names, "%s / ", track->label);
+    else
+      track_names = g_string_append (track_names, track->label);
+  }
+
+  if (mute) {
+    tooltip_str = g_strdup_printf (_("%s: muted"), track_names->str);
+  } else {
+    /* Translator comment: I'm not all too sure if this makes sense
+     * to mark as a translation, but anyway. The string is a list of
+     * selected tracks, the number is the volume in percent. You
+     * most likely want to keep this as-is. */
+    tooltip_str = g_strdup_printf (_("%s: %d%%"), track_names->str,
+        (int) volume);
+  }
+  g_string_free (track_names, TRUE);
+
+  gtk_tooltips_set_tip (applet->tooltips, GTK_WIDGET (applet),
+			tooltip_str, NULL);
+  g_free (tooltip_str);
+
   applet->lock = TRUE;
-  gtk_range_set_value (applet->dock->scale, volume);
+  if (volume != 0)
+    gtk_range_set_value (applet->dock->scale, volume);
   applet->lock = FALSE;
 
   /* update mute status (bonobo) */
@@ -865,7 +1110,7 @@ gnome_volume_applet_refresh (GnomeVolumeApplet *applet)
 static gboolean
 cb_check (gpointer data)
 {
-  gnome_volume_applet_refresh (GNOME_VOLUME_APPLET (data));
+  gnome_volume_applet_refresh (GNOME_VOLUME_APPLET (data), FALSE);
 
   return TRUE;
 }
@@ -882,81 +1127,85 @@ cb_gconf (GConfClient *client,
 {
   GnomeVolumeApplet *applet = data;
   GConfValue *value;
-  const gchar *str;
+  const gchar *str, *key;
   const GList *item;
-  GstMixerTrack *active_track = NULL;
   gboolean newdevice = FALSE;
+  gchar *keyroot;
+  GList *active_tracks;
+
+  active_tracks = NULL;
+
+  keyroot = panel_applet_gconf_get_full_key (PANEL_APPLET (applet), "");
+  key = gconf_entry_get_key (entry);
+  if (strncmp (key, keyroot, strlen (keyroot))) {
+    g_free (keyroot);
+    return;
+  }
+  key += strlen (keyroot);
+  g_free (keyroot);
 
   if ((value = gconf_entry_get_value (entry)) != NULL &&
       (value->type == GCONF_VALUE_STRING) &&
       (str = gconf_value_get_string (value)) != NULL) {
-    if (!strcmp (gconf_entry_get_key (entry),
-		 GNOME_VOLUME_APPLET_KEY_ACTIVE_ELEMENT)) {
+    if (!strcmp (key, GNOME_VOLUME_APPLET_KEY_ACTIVE_ELEMENT)) {
       for (item = applet->elements; item != NULL; item = item->next) {
         gchar *cur_el_str = g_object_get_data (item->data,
 					       "gnome-volume-applet-name");
 
         if (!strcmp (cur_el_str, str)) {
-          GstElement *old_element = GST_ELEMENT (applet->mixer);
+          GstElement *old_element = GST_ELEMENT (applet->mixer),
+		     *new_element = item->data;
 
-          /* change element */
-          gst_element_set_state (item->data, GST_STATE_READY);
-          gst_object_replace ((GstObject **) &applet->mixer, item->data);
-          gst_element_set_state (old_element, GST_STATE_NULL);
-
-          newdevice = TRUE;
-          active_track = gst_mixer_list_tracks (applet->mixer)->data;
-          str = applet->track->label;
+          if (new_element != old_element) {
+            /* change element */
+            if (gst_element_set_state (item->data,
+				       GST_STATE_READY) != GST_STATE_SUCCESS)
+              continue;
+	    
+            /* save */
+            gst_object_replace ((GstObject **) &applet->mixer, item->data);
+            gst_element_set_state (old_element, GST_STATE_NULL);
+            newdevice = TRUE;
+          }
           break;
         }
       }
     }
 
-    if (!strcmp (gconf_entry_get_key (entry),
-		 GNOME_VOLUME_APPLET_KEY_ACTIVE_TRACK) ||
-        newdevice) {
-      for (item = gst_mixer_list_tracks (applet->mixer);
-           item != NULL; item = item->next) {
-        GstMixerTrack *track = item->data;
-
-        if (track->num_channels <= 0)
-          continue;
-
-        if (!strcmp (str, track->label)) {
-          active_track = track;
-          break;
-        } else if (newdevice && !active_track &&
-		   GST_MIXER_TRACK_HAS_FLAG (track,
-					     GST_MIXER_TRACK_MASTER)) {
-          active_track = item->data;
-        }
+    if (!strcmp (key, GNOME_VOLUME_APPLET_KEY_ACTIVE_TRACK) || newdevice) {
+      if (!active_tracks) {
+        active_tracks = select_tracks (GST_ELEMENT (applet->mixer), str, FALSE);
       }
 
-      if (active_track) {
+      if (active_tracks) {
         GtkObject *adj;
-        gint page;
+	GstMixerTrack *first_track;
 
-        g_object_unref (G_OBJECT (applet->track));
-        applet->track = active_track;
-        g_object_ref (G_OBJECT (active_track));
+	/* copy the newly created track list over to the main list */
+	g_list_free (applet->tracks);
+	applet->tracks = g_list_copy (active_tracks);
+
+	first_track = g_list_first (active_tracks)->data;
 
         /* dock */
-        page = (applet->track->max_volume - applet->track->min_volume) / 10;
-        adj = gtk_adjustment_new (0, applet->track->min_volume,
-				  applet->track->max_volume,
-				  (page / 5 > 0) ? page / 5 : 1,
-				  page, 0);
+	adj = gtk_adjustment_new (50, 0, 100, 4, 10, 0);
+	gtk_adjustment_set_value (GTK_ADJUSTMENT (adj), 
+				  gnome_volume_applet_get_volume (applet->mixer, 
+								  first_track));
+
         gnome_volume_applet_dock_change (applet->dock,
 					 GTK_ADJUSTMENT (adj));
         g_signal_connect (adj, "value-changed",
 			  G_CALLBACK (cb_volume), applet);
-
+	
         /* if preferences window is open, update */
-        if (applet->prefs) {
+	if (applet->prefs) {
           gnome_volume_applet_preferences_change (
 	      GNOME_VOLUME_APPLET_PREFERENCES (applet->prefs),
-	      applet->mixer, applet->track);
-        }
+	      applet->mixer, applet->tracks);
+	}
+
+        applet->force_next_update = TRUE;
       }
     }
   }
@@ -1014,20 +1263,24 @@ cb_verb (BonoboUIComponent *uic,
 				  "applet"),
 		"authors",	authors,
 		"translator-credits",	_("translator-credits"),
-		"logo-icon-name",	"gnome-mixer-applet",
+		"logo-icon-name",	"volume-knob",
 		NULL);
 
   } else if (!strcmp (verbname, "Pref")) {
-    if (applet->prefs)
-      return;
+    if (!applet->mixer) {
+      show_no_mixer_dialog (applet);
+    } else {
+      if (applet->prefs)
+        return;
 
-    applet->prefs = gnome_volume_applet_preferences_new (applet->client,
-							 applet->elements,
-							 applet->mixer,
-							 applet->track);
-    g_signal_connect (applet->prefs, "destroy",
-		      G_CALLBACK (cb_prefs_destroy), applet);
-    gtk_widget_show (applet->prefs);
+      applet->prefs = gnome_volume_applet_preferences_new (PANEL_APPLET (applet),
+							   applet->elements,
+							   applet->mixer,
+							   applet->tracks);
+      g_signal_connect (applet->prefs, "destroy",
+		        G_CALLBACK (cb_prefs_destroy), applet);
+      gtk_widget_show (applet->prefs);
+    }
   } else {
     g_warning ("Unknown bonobo command '%s'", verbname);
   }
@@ -1042,10 +1295,11 @@ cb_ui_event (BonoboUIComponent *comp,
 {
   GnomeVolumeApplet *applet = GNOME_VOLUME_APPLET (data);
 
-  if (!strcmp (verbname, "Mute")) {
+  if (!applet->mixer) {
+    show_no_mixer_dialog (applet);
+  } else if (!strcmp (verbname, "Mute")) {
     /* mute will have a value of 4 without the ? TRUE : FALSE bit... */
-    gboolean mute = (GST_MIXER_TRACK_HAS_FLAG (applet->track,
-                        GST_MIXER_TRACK_MUTE)) ? TRUE : FALSE,
+    gboolean mute = applet->state & 1,
              want_mute = !strcmp (state_string, "1") ? TRUE : FALSE;
 
     if (mute != want_mute)
@@ -1054,3 +1308,13 @@ cb_ui_event (BonoboUIComponent *comp,
     g_warning ("Unknown bonobo command '%s'", verbname);
   }
 }
+
+static void
+cb_theme_change (GtkIconTheme *icon_theme,
+		 gpointer data)
+{
+  GnomeVolumeApplet *applet = GNOME_VOLUME_APPLET (data);
+
+  init_pixbufs (applet);
+  gnome_volume_applet_refresh (applet, TRUE);
+}
