@@ -29,16 +29,16 @@
 
 #include <gtk/gtk.h>
 #include <gconf/gconf-client.h>
-#include <libgnome/libgnome.h>
-#include <libgnomeui/libgnomeui.h>
+#include <libgnome/gnome-help.h>
+#include <glade/glade.h>
 
-#include "eel-extension.h"
 #include "trashapplet.h"
 #include "trash-monitor.h"
+#include "xstuff.h"
 
 static GConfClient *client = NULL;
 
-G_DEFINE_TYPE(TrashApplet, trash_applet, PANEL_TYPE_APPLET);
+G_DEFINE_TYPE(TrashApplet, trash_applet, PANEL_TYPE_APPLET)
 
 static void     trash_applet_destroy            (GtkObject         *object);
 
@@ -265,7 +265,7 @@ trash_applet_change_background (PanelApplet               *panel_applet,
 	gtk_widget_set_style (GTK_WIDGET (applet), NULL);
 	rc_style = gtk_rc_style_new ();
 	gtk_widget_modify_style (GTK_WIDGET (applet), rc_style);
-	g_object_unref (rc_style);
+	gtk_rc_style_unref (rc_style);
 
 	switch (type) {
 	case PANEL_NO_BACKGROUND:
@@ -280,6 +280,7 @@ trash_applet_change_background (PanelApplet               *panel_applet,
 			g_object_unref (style->bg_pixmap[GTK_STATE_NORMAL]);
 		style->bg_pixmap[GTK_STATE_NORMAL] = g_object_ref (pixmap);
 		gtk_widget_set_style (GTK_WIDGET (applet), style);
+		g_object_unref (style);
 		break;
 	}
 }
@@ -291,6 +292,7 @@ trash_applet_button_release (GtkWidget      *widget,
 	TrashApplet *applet = TRASH_APPLET (widget);
 
 	if (event->button == 1) {
+		xstuff_zoom_animate(widget, NULL);
 		trash_applet_open_folder (NULL, applet, NULL);
 		return TRUE;
 	}
@@ -482,6 +484,50 @@ update_transfer_callback (GnomeVFSAsyncHandle *handle,
        /* UI updates here */
        TrashApplet *applet = TRASH_APPLET (user_data);
 
+       GladeXML *xml = applet->xml;
+       GtkWidget *location_label;
+       GtkWidget *file_label;
+       GtkWidget *progressbar;
+
+       gdouble fraction = 0.0;
+       gchar *index = "";
+       gchar *total = "";
+       gchar *progress_message = "";
+
+       gchar *from_location = NULL;
+       gchar *file = NULL;
+       GnomeVFSURI *uri;
+
+       if (progress_info->files_total != 0)
+       {
+            fraction = (gulong) progress_info->file_index / (gulong) progress_info->files_total;
+
+	    index = g_strdup_printf ("%i", (gint) progress_info->file_index);
+	    total = g_strdup_printf ("%i", (gint) progress_info->files_total);
+       }
+
+       location_label = glade_xml_get_widget(xml, "location_label");
+       file_label = glade_xml_get_widget(xml, "file_label");
+       progressbar = glade_xml_get_widget(xml, "progressbar");
+
+       gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progressbar), fraction);
+
+       progress_message = g_strdup_printf (_("Removing item %s of %s"), index, total);
+       gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progressbar), progress_message);
+       if (progress_info->source_name != NULL) {
+           uri = gnome_vfs_uri_new(progress_info->source_name);
+
+	   from_location = gnome_vfs_uri_extract_dirname (uri);
+
+	   file = g_strdup_printf (_("<i>Removing: %s</i>"), gnome_vfs_uri_extract_short_name (uri));
+       }
+
+       if (from_location != NULL)
+	       gtk_label_set_markup(GTK_LABEL (location_label), from_location);
+       if (file != NULL) 
+	       gtk_label_set_markup(GTK_LABEL (file_label), file);
+
+
        return 1;
 }
 
@@ -550,11 +596,23 @@ confirm_empty_trash (GtkWidget *parent_view)
 	return response == GTK_RESPONSE_YES;
 }
 
+static void
+on_empty_trash_cancel (GtkWidget *widget, GnomeVFSAsyncHandle **handle)
+{
+	if (handle != NULL) {
+		gnome_vfs_async_cancel ((GnomeVFSAsyncHandle *) handle);
+	}
+		
+	gtk_widget_hide (widget);
+}
+
 static void
 trash_applet_do_empty (BonoboUIComponent *component,
 		       TrashApplet       *applet,
 		       const gchar       *cname)
 {
+	GtkWidget *dialog;
+
 	GnomeVFSAsyncHandle *hnd;
 
 	g_return_if_fail (TRASH_IS_APPLET (applet));
@@ -565,17 +623,30 @@ trash_applet_do_empty (BonoboUIComponent *component,
 	if (!confirm_empty_trash (GTK_WIDGET (applet)))
 		return;
 
+	if (!applet->xml)
+	  applet->xml = glade_xml_new (GNOME_GLADEDIR "/trashapplet.glade", NULL, NULL);
+
+        dialog = glade_xml_get_widget(applet->xml, "empty_trash");
+
+	g_signal_connect(dialog, "response", G_CALLBACK (on_empty_trash_cancel), &hnd);
+
+	gtk_widget_show_all(dialog);
+
 	trash_monitor_empty_trash (applet->monitor,
 				   &hnd, update_transfer_callback, applet);
+
+	gtk_widget_hide(dialog);
+
 }
 
+
 static void
 trash_applet_open_folder (BonoboUIComponent *component,
 			  TrashApplet       *applet,
 			  const gchar       *cname)
 {
 	/* Open the "trash:" URI with gnome-open */
-	gchar *argv[] = { "gnome-open", EEL_TRASH_URI, NULL };
+	gchar *argv[] = { "gnome-open", "trash:", NULL };
 	GError *err = NULL;
 	gboolean res;	
 
@@ -655,7 +726,7 @@ confirm_delete_immediately (GtkWidget *parent_view,
 			    gboolean all)
 {
 	GdkScreen *screen;
-	GtkWidget *dialog, *hbox, *vbox, *image, *label, *button;
+	GtkWidget *dialog, *hbox, *vbox, *image, *label;
 	gchar *str, *prompt, *detail;
 	int response;
 
@@ -747,7 +818,7 @@ trash_applet_drag_data_received (GtkWidget        *widget,
 	GList *source_uri_list, *target_uri_list, *unmovable_uri_list;
 	GnomeVFSResult result;
 
-	list = gnome_vfs_uri_list_parse (selectiondata->data);
+	list = gnome_vfs_uri_list_parse ((gchar *)selectiondata->data);
 
 	source_uri_list = NULL;
 	target_uri_list = NULL;
@@ -755,7 +826,6 @@ trash_applet_drag_data_received (GtkWidget        *widget,
 	for (scan = g_list_first (list); scan; scan = g_list_next (scan)) {
 		GnomeVFSURI *source_uri = scan->data;
 		GnomeVFSURI *trash_uri, *target_uri;
-		gboolean is_local;
 		gchar *source_basename;
 
 		/* find the trash directory for this file */
